﻿using Bsm.Core.Models.Analyze;
using Bsm.Core.Models.Raw;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Bsm.Core.Analyzer
{
    public static class NotePerSecondAnalyzer
    {
        public static NotePerSecondAnalyzeResult Handle(Note[] rawNotes, int bpm)
        {
            // filter by fake
            var realNotes = rawNotes.Where(p => p.CustomData?.Fake != true && (p.Type == 0 || p.Type == 1)).ToArray();
            var beatPerSecond = 60f / bpm;
            // convert to ms
            var msNotes = realNotes.Select(p => new MsNote { Ms = (int)(p.Time * beatPerSecond * 1000), Note = p }).ToArray();
            // group by ms
            var groups = msNotes.GroupBy(p => p.Ms).Select(p => new NoteSegment { Ms = p.Key, Notes = p.Select(q => q.Note).ToArray() }).OrderBy(p => p.Ms).ToArray();

            // calc graph second grid, segment
            var threshold = 4;
            var secondGraph = new int?[msNotes.Max(p => p.Ms / 1000) + 1];
            for (var i = 0; i < secondGraph.Length; i++)
            {
                var start = i * 1000;
                var end = start + 1000;
                var noteCount = msNotes.Count(p => p.Ms >= start && p.Ms < end);
                secondGraph[i] = noteCount < threshold ? null : noteCount;
            }

            var segments = secondGraph.SpliceByCount(5).Select(p => p.All(q => q.HasValue == false) ? null : p.Where(q => q.HasValue).Sum()).ToArray();

            var max = segments.Where(p => p.HasValue).Select(p => p.Value).Max();
            var min = segments.Where(p => p.HasValue).Select(p => p.Value).Min();
            float rng = max - min;
            if (rng == 0) rng = 1;

            var norGraph = segments.Select(p => (p - min) / rng).ToArray();

            // get Real NPS

            var npsNotes = new NoteSegment[groups.Length - 1];

            for (var i = 0; i < groups.Length - 1; i++)
            {
                var item = groups[i];

                var start = item.Ms;

                //find in 1 seconds
                var nList = new List<Note>();
                nList.AddRange(item.Notes);
                var to = start + 1000;
                for (var j = i + 1; j < groups.Length; j++)
                {
                    var next = groups[j];
                    if (next.Ms > to) break;
                    nList.AddRange(next.Notes);
                }

                npsNotes[i] = new NoteSegment { Ms = start, Notes = nList.ToArray() };
            }

            // get NPS
            // calc rest segment

            var peak = groups.OrderByDescending(p => p.Notes.Length).First();
            var avgNps = msNotes.Length / ((msNotes.Max(p => p.Ms) - msNotes.Min(p => p.Ms)) / 1000f);

            var avgNpsRound = (int)Math.Round(avgNps);
            var avgNpsAll = npsNotes.Where(p => p.Notes.Length == avgNpsRound).ToArray();
            var avgNpsInSecond = avgNpsAll.GroupBy(p => p.Ms / 1000).Select(p => p.First()).ToArray();

            var minNps = npsNotes.OrderBy(p => p.Notes.Length).First();
            var minNpsAll = npsNotes.Where(p => p.Notes.Length == minNps.Notes.Length).ToArray();
            var minNpsInSecond = minNpsAll.GroupBy(p => p.Ms / 1000).Select(p => p.First()).ToArray();

            var restsAll = npsNotes.Where(p => p.Notes.Length <= 4).ToArray();
            var restInSecond = restsAll.GroupBy(p => p.Ms / 1000).Select(p => p.First()).ToArray();

            var maxNps = npsNotes.OrderByDescending(p => p.Notes.Length).First();
            var maxNpsAll = npsNotes.Where(p => p.Notes.Length == maxNps.Notes.Length).ToArray();
            var maxNpsInSecond = maxNpsAll.GroupBy(p => p.Ms / 1000).Select(p => p.First()).ToArray();

            return new NotePerSecondAnalyzeResult
            {
                Peak = peak.Notes.Length,

                AvgNps = avgNps,
                AvgNpsInSeconds = avgNpsInSecond.Length,

                MinNps = minNps.Notes.Length,
                MinNpsInSeconds = minNpsInSecond.Length,

                MaxNps = maxNps.Notes.Length,
                MaxNpsInSeconds = maxNpsInSecond.Length,

                RestSeconds = restInSecond.Length,

                Graph = norGraph,
            };
        }

        private class NoteSegment
        {
            public int Ms { get; set; }
            public Note[] Notes { get; set; }

            public override string ToString()
            {
                return $"{TimeSpan.FromMilliseconds(Ms)} - {Notes.Length}";
            }
        }

        private class MsNote
        {
            public int Ms { get; set; }
            public Note Note { get; set; }

            public override string ToString()
            {
                return $"{TimeSpan.FromMilliseconds(Ms)} - {Note}";
            }
        }
    }
}
