﻿using K4os.Compression.LZ4;
using System.IO;

namespace Bsm.Core.Arcache
{
    public class Lz4ArchiveCreator : Lz4ArchiveBase
    {
        private FileStream _blob;
        private MemoryStream _index;

        public Lz4ArchiveCreator(string pathAndBaseName) : base(pathAndBaseName)
        {
            _blob = File.Create(BlobFilePath);
            _index = new MemoryStream();
        }

        public void AppendFile(string path, string archiveName)
        {
            var source = File.ReadAllBytes(path);
            var compressed = Compress(source);

            var arc = new Lz4ArchiveEntry
            {
                FileName = archiveName,
                OriginalSize = source.Length,
                CompressedSize = compressed.Length,
                Offset = _blob.Position,
            };

            lock (this)
            {
                _blob.Write(compressed, 0, compressed.Length);
                arc.WriteToIndex(_index);
            }
        }

        public void Close()
        {
            _blob?.Close();
            var bufRaw = _index.ToArray();
            var bufCompress = CompressPickle(bufRaw);
            File.WriteAllBytes(IndexFilePath, bufCompress);
        }
    }
}
