﻿using Bsm.Core.Models.Raw;
using Newtonsoft.Json;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

namespace Bsm.Core.Loader
{
    public static class CustomLevelLoader
    {
        public static CustomLevelModel FromDirectory(string dir)
        {
            var infoFilePath = Path.Combine(dir, "info.dat");

            using var toHashData = new MemoryStream();
            var infoData = File.ReadAllBytes(infoFilePath);
            toHashData.Write(infoData, 0, infoData.Length);

            var infoJson = Encoding.UTF8.GetString(infoData);
            var infoObj = JsonConvert.DeserializeObject<CustomLevelModel>(infoJson);

            foreach (var beatmap in infoObj.DifficultyBeatmapSets.SelectMany(p => p.DifficultyBeatmaps))
            {
                var beatmapFilePath = Path.Combine(dir, beatmap.BeatmapFilename);
                var beatmapData = File.ReadAllBytes(beatmapFilePath);
                toHashData.Write(beatmapData, 0, beatmapData.Length);
                var beatmapJson = Encoding.UTF8.GetString(beatmapData);

                beatmap.Content = JsonConvert.DeserializeObject<DifficultyBeatmapContent>(beatmapJson);
            }

            infoObj.Hash = toHashData.ToArray().Sha1Hex();

            return infoObj;
        }

        public static CustomLevelModel FromZip(string path)
        {
            using var zip = ZipFile.OpenRead(path);

            using var toHashData = new MemoryStream();
            var infoData = zip.ReadEntry("info.dat");
            toHashData.Write(infoData, 0, infoData.Length);

            var infoJson = Encoding.UTF8.GetString(infoData);
            var infoObj = JsonConvert.DeserializeObject<CustomLevelModel>(infoJson);

            foreach (var beatmap in infoObj.DifficultyBeatmapSets.SelectMany(p => p.DifficultyBeatmaps))
            {
                var beatmapData = zip.ReadEntry(beatmap.BeatmapFilename);
                toHashData.Write(beatmapData, 0, beatmapData.Length);
                var beatmapJson = Encoding.UTF8.GetString(beatmapData);

                beatmap.Content = JsonConvert.DeserializeObject<DifficultyBeatmapContent>(beatmapJson);
            }

            infoObj.Hash = toHashData.ToArray().Sha1Hex();

            return infoObj;
        }

        private static byte[] ReadEntry(this ZipArchive zip, string path)
        {
            var entry = zip.Entries.FirstOrDefault(p => p.FullName.ToLower() == path.ToLower());
            if (entry == null) throw new FileNotFoundException("entry not found", path);
            var buf = new byte[entry.Length];
            var ms = new MemoryStream(buf, 0, buf.Length, true, true);
            using var zs = entry.Open();
            zs.CopyTo(ms);
            return ms.GetBuffer();
        }

        private static string Sha1Hex(this byte[] bytes)
        {
            using var sha1 = new SHA1Managed();
            return string.Join("", sha1.ComputeHash(bytes).Select(p => p.ToString("x2")));
        }
    }
}
